/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Global exception class stuff.
**
*****************************************************************************/

#ifndef FANCY_EXCEPTION_H
#define FANCY_EXCEPTION_H

#include <fancybaselog.h>
#include <exception>

namespace Fancy
{
  class FANCYBASE_API Exception : public std::exception
    {
    public:
      /**
       * fallback contructor generating an 'unknown exception'.
       */
      Exception() throw();

      Exception (int facility,
                 int priority,
                 const char *fmt, ...) throw()
#ifdef __GNUG__
        __attribute__((format(printf,4,5)))
#endif
        ;

      Exception (int facility,
                 int priority,
                 const char *fmt, va_list ap) throw();

      Exception (const Exception &) throw();
      Exception (const std::exception &) throw();
      Exception &operator= (const Exception &) throw();

      ~Exception() throw();

      static void log(const std::exception &e) throw();

      static void logBacktrace(fancylog_service *service,
              int facility,
              int priority,
              const char *msg) throw();

      virtual void log() const throw();
      const char *what() const throw();
      fancylog_service *getService() const throw();
      int getFacility() const throw();
      int getPriority() const throw();

    protected:
      Exception (fancylog_service *service,
                 int facility,
                 int priority ) throw();

      void appendMessageV(const char *fmt, va_list ap) throw();
#ifdef __GNUG__
      __attribute__((format(printf,2,3)))
#endif
      void appendMessage(const char *fmt, ...) throw();

    private:
      fancylog_service *svc;
      int fac;
      int pri;
      void *msg;
    };

  class FANCYBASE_API CException : public Exception
    {
    public:
      CException (int facility,
                  int priority,
                  const char *fmt, ...) throw()
#ifdef __GNUG__
      __attribute__((format(printf,4,5)))
#endif
        ;

      CException (int facility,
                  int priority,
                  int _err,
                  const char *fmt, ...) throw()
#ifdef __GNUG__
      __attribute__((format(printf,5,6)))
#endif
        ;

      ~CException() throw();

      int getErrno() const throw();
      const char *getStrError() const throw();

      virtual bool isInterrupt() const throw();

    protected:
      CException (fancylog_service *service,
                  int facility,
                  int priority) throw();

      CException (fancylog_service *service,
                  int facility,
                  int priority,
                  int _err) throw();

      void appendCMessageV(const char *msg, va_list ap) throw ();
#ifdef __GNUG__
      __attribute__((format(printf,2,3)))
#endif
      void appendCMessage(const char *msg, ...) throw ();

    private:
      int err;
#ifdef WIN32
      char *strerr;
#endif
    };

  /**
   * An exception thrown in protocol handlers and at other occasions,
   * where more input data is expected or the output data could not be
   * fully written to a stream.
   */
  class FANCYBASE_API EOFException : public Exception
    {
    public:
      EOFException (int facility,
                  int priority,
                  const char *fmt, ...) throw()
#ifdef __GNUG__
      __attribute__((format(printf,4,5)))
#endif
        ;

      ~EOFException() throw();

    protected:
      EOFException (fancylog_service *service,
                  int facility,
                  int priority) throw();
    };
} // namespace Fancy

#endif // ! FANCY_EXCEPTION_H
