/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Wrapper class for a file handle.
**
*****************************************************************************/

#ifndef FANCYBASE_FILE_HANDLE_H
#define FANCYBASE_FILE_HANDLE_H

#include <fancypollablechannel.h>

namespace Fancy {
#ifdef WIN32
      typedef void *file_handle_t;
#else
      typedef int file_handle_t;
#endif

  class FANCYBASE_API FileHandle : public virtual PollableChannel
  {
  public:
      typedef Fancy::ObjPtr<FileHandle> ptr_t;

      enum Mode { ReadOnly, WriteOnly, ReadWrite };
      enum InheritFlag { InheritNone, InheritRead, InheritWrite, InheritBoth };
      enum StdHandle { StdIn, StdOut, StdErr };

      FileHandle() {}

      /**
       * Return a new instance for your operating system representing a standard
       * file handle.
       */
      static ptr_t newInstance(StdHandle stdHandle);

      /**
       * Take over a file handle opened by native operating system facilities.
       */
      static ptr_t newInstance(file_handle_t fd, Mode mode);

      /**
       * Open a regular file. If @c mode is either @c WriteOnly or @c ReadWrite,
       * create the file if it does not exist.
       */
      static ptr_t newInstance(const char *path, Mode mode);

      /**
       * Open a regular file. If @c mode is either @c WriteOnly or @c ReadWrite,
       * create the file if it does not exist.
       */
      static ptr_t newInstance(const std::string &path, Mode mode);

#ifndef WIN32
      /**
       * Open a regular file.
       * If @c mode is either @c WriteOnly or @c ReadWrite,
       * create the file with the given UNIX permissions, if it does not exist.
       */
      static ptr_t newInstance(const char *path, Mode mode, int perms);

      /**
       * Open a regular file.
       * If @c mode is either @c WriteOnly or @c ReadWrite,
       * create the file with the given UNIX permissions, if it does not exist.
       */
      static ptr_t newInstance(const std::string &path, Mode mode, int perms);
#endif
      /**
       * Open a pipe with a given inheritance characteristics.
       *
       * @param readHandle This pointer is set to a new instance upon success and will
       *                   point to the read end of the pipe.
       * @param writeHandle This pointer is set to a new instance upon success and will
       *                    point to the write end of the pipe.
       * @param flag Defines whether one or two of the returned handles will be inherited
       *             by parent processes.
       */
      static void openPipe(ptr_t &readHandle, ptr_t &writeHandle, InheritFlag flag);

      /**
       * Disclose the operating system file handle.
       */
      virtual file_handle_t getHandle() const = 0;

  protected:

      virtual ~FileHandle();
  };

}

#endif

