/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCY_MIMETYPE_RESOLVER_H_
#define FANCY_MIMETYPE_RESOLVER_H_

#include <fancyobject.h>


namespace Fancy {

/**
 * A mime type resolver helps find mime types for file extensions and vice versa.
 * It facilitates the OS specific core list and allows reading additional files
 * that follow the /etc/mime.types style.
 */
class FANCYBASE_API MimeTypeMapping: public virtual Fancy::Object {
protected:
    virtual ~MimeTypeMapping();
public:
    typedef ObjPtr<MimeTypeMapping> ptr_t;

    /**
     * Return a new MimeTypeMapping instance.
     *
     * In Linux, readOSList triggers readMimeTypesFile("/etc/mime.types").
     * An eventual ~/.mime.types file is not read automatically.
     *
     * In Windows, readOSList triggers reading the ac. Registry entries are read.
     */
    static ptr_t newInstance(bool readOSList);

    /**
     * If the specified file exists, it is read, overruling anything read before,
     * hence allowing application config files to overrule OS wide configurtion.
     *
     * The file is supposed to conform to the syntax of Linux' /etc/mime.types file.
     * Eventual lines with syntax errors are just ignored.
     * Lines that only contain a mimeType, but no file name extension, are ignored, too.
     *
     * Returns the number of entries read correctly and applied to the internal map.
     */
    virtual int readMimeTypesFile(const std::string & fileName) = 0;

    /**
     * Returns the mime type for the extension if a mapping is known, empty string otherwise.
     * The extension is case sensitive, usually all are lower case.
     */
    virtual std::string getMimeTypeForExtension(const std::string & extension) const = 0;

    virtual std::string getExtensionForMimeType(const std::string & mimeType) const = 0;

    virtual int getNrMimeTypes() const = 0;

    virtual int getNrExtensions() const = 0;


    // evtl. add fancylaunch stuff

};

}

#endif /* FANCY_MIMETYPE_RESOLVER_H_ */
