/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYQUEUE_H_
#define FANCYQUEUE_H_

#include <fancylist.h>

namespace Fancy {

class Iterator;

/*!
 * An interface for an untyped queue of objects.
 * This is pretty much like \c java.util.Queue and is used in favor
 * to a template in order to avoid code-bloat and to allow a generic
 * deserializer to be implemented.
 */
class FANCYBASE_API Queue: public virtual List {
public:
    typedef ObjPtr<Queue> ptr_t;

    virtual void addFront(const ObjPtr<Object> &o) = 0;
    virtual ObjPtr<Object> popFront() = 0;

    static ObjPtr<Queue> newInstance();
    static ObjPtr<Queue> newInstance(getSerialID_s_t getSerialID_s, getSerialVersionUID_s_t getSerialVersionUID_s);

    template <class T> static ObjPtr<Queue> newTypedInstance() {
        return Queue::newInstance(T::getSerialID_s,T::getSerialVersionUID_s);
    }

protected:
    Queue() {}
    virtual ~Queue();
};

}

#endif /* FANCYQUEUE_H_ */
